<?php

/**
 * @version
 *		1.0
 * @copyright
 *		Weizmann Institute for Science.
 * @author
 *		Avner Sass http://www.klido.net
 *
 */
class TNSClient {

	/// TNS's API URL
	protected static $baseAPIUrl = 'https://sandbox.wis-tns.org/api/';

	/// General configurable options
	protected $generalOptions = array();
	
	/// TNS MArker
	protected $tns_marker;
	
	/// Time till rate-limit reset
	protected $reset;
	
	/// Response headers.
	protected $response_headers = array();


	/**
	 * Constructor
	 *
	 * @param array $extra_opts
	 *	 extra options
	 */
	public function __construct($extra_options = array()) {
		$this->generalOptions = array_merge((array)$extra_options);
	}


	/**
	 *
	 */
	public function getSearch($options = array()) {
		$feed_url = $this->buildUrl('get/search');
		$feed_parameters = $this->buildParameters(array('data' => $options));
		return $this->getFeed($feed_url, $feed_parameters);
	}

  /**
	 *
	 */
	public function getObject($options = array()) {
		$feed_url = $this->buildUrl('get/object');
		$feed_parameters = $this->buildParameters(array('data' => $options));
		return $this->getFeed($feed_url, $feed_parameters);
	}

  /**
	 *
	 */
	public function getFile($options = array()) {
		$feed_url = $options['file'];
		$feed_parameters = $this->buildParameters(array());
		return $this->getFeed($feed_url, $feed_parameters);
	}

	/**
	 * Build and clean the URL
	 *
	 * @param $resource
	 *	 resource part of the URL
	 *
	 * @return String containing URL.
	 */
	protected function buildUrl($resource) {
		if (!empty($resource)) {
			return self::$baseAPIUrl . $resource;
		}
		else {
			throw new Exception('Error: resource must be set.');
		}
	}

	/**
	 * Build and clean the URL
	 *
	 * @param $parameters
	 *
	 * @return String containing URL.
	 */
	protected function buildParameters($parameters) {
		return array_merge($this->getGeneralOptions(), $parameters);
	}

 /**
	* Get feed from API.
	*
	* @param $feed_url
	*	 URL of the requested data.
	*
	* @return Object containing requested result.
	*/
	protected function getFeed($feed_url, $parameters) {

		$error_status = '';
		$feed = '';

		//open connection
		$ch = curl_init();

		$options = array(
			CURLOPT_URL				      => $feed_url,
			CURLOPT_RETURNTRANSFER  => TRUE,
			CURLINFO_HEADER_OUT		  => TRUE,
			CURLOPT_HEADER			    => TRUE,
			CURLOPT_SSL_VERIFYPEER 	=> FALSE,
			CURLOPT_POST			      => TRUE,
	    CURLOPT_USERAGENT		    => $this->tns_marker,
			CURLOPT_POSTFIELDS		  => $parameters,
		);

		curl_setopt_array($ch, $options);

		$result = curl_exec($ch);

		$header_info = curl_getinfo($ch, CURLINFO_HEADER_OUT);
		$status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		$header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);

		$http_response_header = substr($result, 0, $header_size);
		$body = substr($result, $header_size);
		curl_close($ch);

		$headers = explode("\r", str_replace("HTTP/1.1 100 Continue\x0d\x0a\x0d\x0a", '', $http_response_header));
		foreach($headers as $header){
		  $header = explode(': ', $header);
		  if(isset($header[1])) {
		    $this->response_headers[trim($header[0])] = trim($header[1]);
		  }
		}

		// Retrieve HTTP status code
		list($version, $status, $msg) = explode(' ', $headers[0], 3);
		
		$json = json_decode($body, TRUE);
		$status_code = $json != FALSE  ? $json['id_code'] : $status_code;
		
		// Check the HTTP Status code
		switch($status_code) {
			case 200:
				
				return !empty($json) ? $json : $body;
				break;

			case 304:
				$error_status = 'Error 304: Not Modified: There was no new data to return.';
				break;

			case 400:
				$error_status = 'Error 400: Bad Request: The request was invalid. An accompanying error message will explain why.';
				break;
		
			case 401:
			  $error_status = 'Error 401: Unauthorized.';
			  break;

			case 403:
				$error_status = 'Error 403: Forbidden: The request is understood, but it has been refused. An accompanying error message will explain why';
				break;

			case 404:
				$error_status = 'Error 404: Not Found: The URI requested is invalid or the resource requested, such as a category, does not exists.';
				break;

			case 500:
				$error_status = 'Error 500: Internal Server Error: Something is broken.';
				break;

			case 503:
				$error_status = 'Error 503: Service Unavailable.';
				break;

			default:
				$error_status = 'Undocumented error';
				break;

		}
		
		var_dump(array($error_status, $feed_url, $msg, $json));
		return FALSE;
	}

	/*
	 * Get the generalOptions array
	 */
	public function getGeneralOptions() {
		return $this->generalOptions;
	}

	/**
	 * Sets the generalOptions array
	 *
	 * @param array $generalOptions
	 */
	public function setGeneralOptions($general_options) {
		$this->generalOptions = $general_options;
	}
	
	/**
	 *
	 */
	public function setTnsMarker($tns_marker){
	  $this->tns_marker = $tns_marker;
	}
	
	/**
	 *
	 */
	public function getReponseHeaders(){
	  return $this->response_headers;
	}
	
	/**
	 *
	 */
	public function isReachedRateLimit(){
	  foreach($this->response_headers as $header => $value){
	    if(self::ends_with($header, 'rate-limit-remaining') && $value == 0){
	      return $this->response_headers[str_replace('remaining', 'reset', $header)];
	    }
	  }
	  return NULL;
	}
	
	/**
	 *
	 */
	protected static function ends_with($string, $test) {
	  $strlen  = strlen($string);
	  $testlen = strlen($test);
	  if ($testlen > $strlen) return FALSE;
	  return substr_compare($string, $test, $strlen - $testlen, $testlen) === 0;
	}

}


/** Helper functions **/
/**
 * Find key in multi-dimentional array and return its value.
 */
function findKey($haystack, $needle) {

	if (array_key_exists($needle, $haystack)) {
		return $haystack[$needle];
	}
	foreach ($haystack as $k => $v) {
		if (!is_array($v)) {
			continue;
		}
		else if (array_key_exists($needle, $v)) {
			return $v[$needle];
		}
		else{
			$result = findKey($v, $needle);
			if($result !== FALSE) {
				return $result;
			}
		}
	}
	return false;
}

define('API_KEY', 'YOUR BOT API KEY');

// Send search parameters.
$feed_handler = new TNSClient(array('api_key' => API_KEY));
$feed_handler->setTnsMarker('tns_marker{"tns_id": "YOUR BOT ID ", "type": "bot", "name": "YOUR BOT NAME"}"');
$json = array(
  "ra"            => "05:24:18.0",
  "dec"           => "+09:10:37.0",
  "radius"        => "5",
  "units"         => "arcsec",
  "objname"       => "",
  "internal_name" => ""
);
while (TRUE) {
  $reply = $feed_handler->getSearch(json_encode($json)); 
  if($reply == FALSE ){
    die("Error getting search results\n");
  }

  // Print request results.
  $headers = $feed_handler->getReponseHeaders();
  $print = [];
  $print[] = 'Limit: ' . $headers['x-rate-limit-limit'];
  $print[] = 'Remaining: ' . $headers['x-rate-limit-remaining'];
  $print[] = 'Reset: ' . $headers['x-rate-limit-reset'];
  
  if(isset($headers['x-cone-rate-limit-limit'])){
    $print[] = 'Cone Limit: ' . $headers['x-cone-rate-limit-limit'];
    $print[] = 'Cone Remaining: ' . $headers['x-cone-rate-limit-remaining'];
    $print[] = 'Cone Reset: ' . $headers['x-cone-rate-limit-reset'];
  }
  print_r(implode(' | ', $print) . "\n");
  
  $reset = $feed_handler->isReachedRateLimit();
  if($reset != NULL){
    // If rate-limit reached (x-rate-limit-remaining=0) sleeping for x-rate-limit-reset sec
    echo 'Rate limit reached' . "\n";
    break;
  }
}
echo 'Sleep ' . $reset . " seconds\n";
sleep($reset);

// Keep sending
$reply = $feed_handler->getSearch(json_encode($json));
if($reply == FALSE ){
  die("Error getting search results\n");
}
$headers = $feed_handler->getReponseHeaders();
$print = [];
$print[] = 'Limit: ' . $headers['x-rate-limit-limit'];
$print[] = 'Remaining: ' . $headers['x-rate-limit-remaining'];
$print[] = 'Reset: ' . $headers['x-rate-limit-reset'];

if(isset($headers['x-cone-rate-limit-limit'])){
  $print[] = 'Cone Limit: ' . $headers['x-cone-rate-limit-limit'];
  $print[] = 'Cone Remaining: ' . $headers['x-cone-rate-limit-remaining'];
  $print[] = 'Cone Reset: ' . $headers['x-cone-rate-limit-reset'];
}
print_r(implode(' | ', $print) . "\n");
echo 'Test again' . "\n";


$obj_request = array(
    "photometry"  => "1",
    "spectra"     => "1"
);

// Looping over retrieved objects list
foreach($reply['data']['reply'] as $delta => $objname) {
  $obj_request["objname"] = $objname['objname'];
  
  // Retrieving object's data (photometry,spectra)
  $object = $feed_handler->getObject(json_encode($obj_request));
  print_r($object);
  
  // Check if key is present.
  $obj_find = findKey($reply, 'id_code');
  
  if($obj_find > 400) {
    print_r('General error: ' . $reply['id_message']);
  }
  else if($obj_find == 400) {
    print_r('Bad request. Please check feedback for more information');
  }
  else {
    foreach($object['data']['reply']['spectra'] as $index => $spectra) {
      if(isset($spectra['asciifile']) && !empty($spectra['asciifile'])) {
        $ascii = $feed_handler->getFile(array('file' => $spectra['asciifile']));
        $filename = basename($spectra['asciifile']);
        file_put_contents($filename, $ascii);
      }
    }
  }
}


?>
